package a3rLoader;

import java.io.*;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileNameExtensionFilter;

public class Loader {

	public static void main(String[] args) {
		Loader loader = new Loader();
		loader.start();
	}
	
	public void start() {
		Model particles = loadFile();
		if (particles == null) System.exit(0);
	}
	
	/**
	 * ,   ,  
	 * */
	public static Model loadFile() {
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setFileFilter(new FileNameExtensionFilter("*.a3r", "a3r"));
		fileChooser.setCurrentDirectory(new File("."));
		fileChooser.showDialog(null, "Load a3r file");
		File file = fileChooser.getSelectedFile();
		if (file == null) {
			System.out.println("  ");
			System.exit(0);
		}
		
		return loadFile(file);
		

	}

	private static Model loadFile(File file) {
		FileInputStream fis;
		BufferedInputStream bis = null;
		Particle[] particles = null;
		Model model = null;
		float xMin = 0, xMax = 0, yMin = 0, yMax = 0, zMin = 0, zMax = 0;
		class FileTypeException extends Exception {
			private static final long serialVersionUID = 1L;
			public FileTypeException(String str){
				super(str);
			}
		}
		try {
			String fileType;
			int count;
			int dataStart;
			String version;
			double r;
			int count_1;
			model = new Model();
			
			System.out.println(" a3r  \"" + file.getName() + "\"");
			
			fis = new FileInputStream(file);
			bis = new BufferedInputStream(fis);
			
			//fileType
			byte[] bytes4 = new byte[4];
			ByteBuffer bBuf = ByteBuffer.wrap(bytes4);
			bBuf.order(ByteOrder.BIG_ENDIAN);
			bis.read(bytes4);
			fileType = ("" + (char)bBuf.get() + (char)bBuf.get() + (char)bBuf.get() + (char)bBuf.get()).trim();
			if (!fileType.equals("a3r")) {throw new FileTypeException("It's not an a3r file");}
			
			//count
			bis.read(bytes4);
			bBuf.flip();
			bBuf.order(ByteOrder.LITTLE_ENDIAN);
			count = bBuf.getInt();
			
			//dataStart
			bis.read(bytes4);
			bBuf.flip();
			bBuf.order(ByteOrder.LITTLE_ENDIAN);
			dataStart = bBuf.getInt();
			
			//version
			byte[] bytes12 = new byte[12];
			bBuf = ByteBuffer.wrap(bytes12);
			bBuf.order(ByteOrder.BIG_ENDIAN);
			bis.read(bytes12);
			char[] chars = new char[12];
			for (int i = 0; i < bytes12.length; i++) {
				chars[i] = (char)bBuf.get();
			}
			version = String.valueOf(chars);

			//r
			byte[] bytes8 = new byte[8];
			bBuf = ByteBuffer.wrap(bytes8);
			bBuf.order(ByteOrder.LITTLE_ENDIAN);
			bis.read(bytes8);
			r = bBuf.getDouble();
			
			//count_1
			bis.read(bytes4);
			bBuf = ByteBuffer.wrap(bytes4);
			bBuf.order(ByteOrder.LITTLE_ENDIAN);
			count_1 = bBuf.getInt();
			
			//   
			int offset = dataStart - 4 - 4 - 4 - 12 - 8 - 4;
			bis.read(new byte[offset]);
			
			//  
			bBuf = ByteBuffer.wrap(bytes4);
			bBuf.order(ByteOrder.LITTLE_ENDIAN);
			particles  = new Particle[count];
			
			System.out.println("     ");
			for (int i = 0; i < particles.length; i++) {
				//  ,   LITTLE_ENDIAN
//				bis.read(bytes12);
//				particles[i].setVector(bBuf.getFloat(), bBuf.getFloat(), bBuf.getFloat());
				
				// ,  getFloat()   -   ,   ++  float
				
				bis.read(bytes4);
				float x = bBuf.getFloat();
				bBuf.flip();
				
				bis.read(bytes4);
				float y = bBuf.getFloat();
				bBuf.flip();
				
				bis.read(bytes4);
				float z = bBuf.getFloat();
				bBuf.flip();
				
				if (x < xMin) xMin = x;
				if (x > xMax) xMax = x;
				if (y < yMin) yMin = y;
				if (y > yMax) yMax = y;
				if (z < zMin) zMin = z;
				if (z > zMax) zMax = z;
				
				particles[i] = new Particle(new Vector3D(x, y, z));
			}
			
			bBuf.order(ByteOrder.BIG_ENDIAN);
			if (bis.available() != 0) {
				System.out.println("     ");
				model.colorsLoaded = true;
				for (int i = 0; i < particles.length; i++) {
					
					bis.read(bytes4);
					int color = (bBuf.getInt() >> 8);
					bBuf.flip();
					
					particles[i].setColor(color);
					
				}
			}
			
			bBuf.order(ByteOrder.LITTLE_ENDIAN);
			if (bis.available() != 0) {
				System.out.println("     ");
				model.radiusesLoaded = true;
				for (int i = 0; i < particles.length; i++) {
					
					bis.read(bytes4);
//					int radius = bBuf.getInt();
					float radius = bBuf.getFloat();
					bBuf.flip();
					
					particles[i].setRadius(radius);
					
				}
			} else {
				for (int i = 0; i < particles.length; i++) {
					particles[i].setRadius((float)r);
				}
			}
			
			System.out.println("yMin = " + yMin + "   yMax = " + yMax);
			model.setParticles(particles);
			model.setBorders(xMin, xMax, yMin, yMax, zMin, zMax);
			
			System.out.println(" a3r  \"" + file.getName() + "\"  " + count + " ");
			if (bis.available() != 0) {
				System.out.println("!!!  / , unprocessed bytes = " 
						+ bis.available());
			}
			bis.close();
			
		} catch (FileNotFoundException e) {e.printStackTrace();
		} catch (IOException e) {e.printStackTrace();
		} catch (FileTypeException e) {e.printStackTrace();
		} finally {
			try {
				if (bis != null) bis.close();
			} catch (IOException e) {e.printStackTrace();}
		}
		return model;
	}
	
	
}
